/***********************************************************
 * Hajime no Ippo --- file : hjime.cpp
 ***********************************************************/
#include <ode/ode.h>
#include <drawstuff/drawstuff.h>

#define MAX_CONTACTS 4 // $B:GBg$N>WFM8!=P2DG=?t(B
static dWorldID world; // $B@$3&(BID
static dSpaceID space; // $B%9%Z!<%9(BID
static dJointGroupID contactgroup;
                       // $B>WFMMQ$N%8%g%$%s%H%0%k!<%W(B

static dBodyID box;    // $B9dBN$N(BBodyID
static dGeomID geom;   // $B9dBN$N(BGeomID
dReal  box_sizes[3] = { 1.0, 1.0, 1.0 };
                       // $B9dBN$N%5%$%:(B

#ifdef dDOUBLE
  #define dsDrawBox dsDrawBoxD
#endif

// $B>WFM8!=PMQ4X?t(B
static void nearCallback( void *data, dGeomID o1, dGeomID o2 )
{
  dBodyID b1 = dGeomGetBody( o1 ); // $BJ*BN(B1
  dBodyID b2 = dGeomGetBody( o2 ); // $BJ*BN(B2

  if ( b1 && b2 && dAreConnectedExcluding( b1, b2, dJointTypeContact ) )
    return; // $B>WFMBP>]$G$J$$J*BN$N>WFM$O$O$:$9(B

  dContact contact[MAX_CONTACTS];
  for ( int i=0; i<MAX_CONTACTS; i++ )
  {
    // $BJ*BNF1;N$N@\?(;~$N%Q%i%a!<%?@_Dj(B
    contact[i].surface.mode = dContactBounce | dContactSoftCFM;
    contact[i].surface.mu = dInfinity;   // $BK`;$78?t(B
    contact[i].surface.bounce = 0.1;     // $BH?H/78?t(B
    contact[i].surface.soft_cfm = 0.01;  // CFM$B@_Dj(B
  }

  // $B>WFM8!=P(B
  int numc = dCollide( o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof( dContact ) ); 
  if ( numc > 0 )
  {
    for ( int i=0; i<numc; i++ )
    {
      // $B>WFM$NH/@8(B
      dJointID c = dJointCreateContact( world, contactgroup, contact+i );
                                 // $B@\?(%8%g%$%s%H@8@.(B
      dJointAttach( c, b1, b2 ); // $B@\?(%8%g%$%s%H@\B3(B
    }
  }
}

// start simulation - set viewpoint
static void start()
{
  static float xyz[3] = { 0.f, 5.f, 3.f };
  static float hpr[3] = { -90.f, -15.f, 0.f };

  dsSetViewpoint( xyz, hpr ); // $B%+%a%i0LCV$HJ}8~@_Dj(B
}

// simulation loop
static void simLoop( int pause )
{
  dSpaceCollide( space, 0, &nearCallback ); // $B>WFM8!=P(B

  // Ctl+p $B$,2!$5$l$?$i(Bif$B$KF~$i$J$$(B
  if (!pause)
  {
    dWorldStep( world, 0.01 ); // $B@$3&$r?J$a$k(B
  }

  dJointGroupEmpty( contactgroup ); // $B@\?($N=i4|2=(B

  // $B9dBN$NI=<((B
  dsSetColor( 1, 1, 0 ); // RGB Color
  dsDrawBox( dBodyGetPosition( box ),
             dBodyGetRotation( box ),
             box_sizes );
}


int main( int argc, char* argv[] )
{
  // setup pointers to drawstuff callback functions
  dsFunctions fn;
  fn.version = DS_VERSION;
  fn.start   = &start;
  fn.step    = &simLoop;
  fn.command = 0;
  fn.stop    = 0;
  fn.path_to_textures = "../drawstuff/textures";

  // create world
  world = dWorldCreate();         // $B@$3&@8@.(B
  space = dHashSpaceCreate( 0 );  // $B%9%Z!<%9(B($B>WFM@$3&(B)$B@8@.(B
  contactgroup = dJointGroupCreate( 0 );     // $B>WFMMQ$N%8%g%$%s%H%0%k!<%W:n@.(B
  dCreatePlane( space, 0, 0, 1, 0 );         // $BCOLL@8@.(B
  dWorldSetGravity( world, 0.0, 0.0, -9.8 ); // $B@$3&$K=ENO$r@8@.(B(z$BJ}8~$K(B-9.8m/s/s)

  // create box
  box = dBodyCreate( world );     // $B@$3&$K9dBN$r:n@.(B
  dReal pos[3] = { 0.0, 0.0, 3.0 };
  dBodySetPosition( box, pos[0], pos[1], pos[2] );
                                  // $B9dBN$N0LCV$r@_Dj(B
  geom = dCreateBox( space, box_sizes[0], box_sizes[1], box_sizes[2] );
                                  // $B%9%Z!<%9$KBg$-$5(Bbox_sizes$B$NN)J}BN$r@8@.(B
  dGeomSetBody( geom, box );      // geom$B$H(Bbox$B$N4X78$E$1(B

  // starting simulation
  dsSimulationLoop( argc, argv, 320, 240, &fn );
                                  // $B%7%_%e%l!<%7%g%s3+;O(B(simLoop$B$X(B)
                                  // + $B%&%#%s%I%&%5%$%:@_Dj(B

  dJointGroupDestroy( contactgroup ); // $B%8%g%$%s%H%0%k!<%W$NGK4~(B
  dSpaceDestroy( space );         // $B%9%Z!<%9$NGK4~(B
  dWorldDestroy( world );         // $B@$3&$NGK2u(B
  return 0;
}

